package main;

import java.sql.Date;
import java.text.SimpleDateFormat;
import java.util.Random;
import java.util.TimeZone;

import com.google.gson.JsonObject;
import com.huawei.iota.iodev.IodevService;
import com.huawei.iota.iodev.datatrans.DataTransService;
import com.huawei.iota.util.IotaMessage;
import com.huawei.iota.util.MyObserver;

import bean.GatewayInfo;

public class AgentLiteDataTrans implements MyObserver {
	
	private static AgentLiteDataTrans instance = new AgentLiteDataTrans();
	private static final String MSG_TIMESTAMP_FORMAT = "yyyyMMdd'T'HHmmss'Z'";
	
    public static AgentLiteDataTrans getInstance() {
        return instance;
    }
    
    public void gwDataReport() {
        System.out.println(" =============   gwDataReport! ============== ");
        int cookie;
        Random random = new Random();
        cookie = random.nextInt(65535);

        String deviceId = GatewayInfo.getDeviceID();
      
        JsonObject data = new JsonObject();
        data.addProperty("storage", "10240");
        data.addProperty("usedPercent", "20");
        DataTransService.dataReport(cookie, null, deviceId, "Storage", data.toString());
    }
    
    public void gwDataReportByMqttDataPub() {
        System.out.println(" =============   gwDataReportByMqttDataPub! ============== ");
        int cookie;
        Random random = new Random();
        cookie = random.nextInt(65535);

        String deviceId = GatewayInfo.getDeviceID();
        
        JsonObject headerData = new JsonObject();
        headerData.addProperty("method", "PUT");
        String fromStr = "/device/"+deviceId+"/services/Storage";
        String toStr = "/data/v1.1.0/devices/"+deviceId+"/services/Storage";
        headerData.addProperty("from", fromStr);
        headerData.addProperty("to", toStr);
        
        headerData.addProperty("access_token", GatewayInfo.getAccessToken());

        SimpleDateFormat df = new SimpleDateFormat(MSG_TIMESTAMP_FORMAT);
        df.setTimeZone(TimeZone.getTimeZone("UTC"));
        String curTime = df.format(new Date(System.currentTimeMillis()));
        headerData.addProperty("timestamp", curTime);
        headerData.addProperty("eventTime", curTime);
        
        JsonObject bodyData = new JsonObject();
        bodyData.addProperty("storage", "10240");
        bodyData.addProperty("usedPercent", "18");
        
        JsonObject mqttMsg = new JsonObject();
        mqttMsg.add("header", headerData);
        mqttMsg.add("body", bodyData);
        
        DataTransService.mqttDataPub(cookie, "/cloud/signaltrans/v2/categories/data", 1, mqttMsg.toString().getBytes());
    }
        
    public void subDevDataReport() {
        System.out.println(" =============   subDevDataReport! ============== ");
        int cookie;
        Random random = new Random();
        cookie = random.nextInt(65535);

        String deviceId = GatewayInfo.getSensorId();
      
        DataTransService.dataReport(cookie, null, deviceId, "Battery", "{\"batteryLevel\":98}");
        DataTransService.dataReport(cookie, null, deviceId, "Motion", "{\"motion\":\"DETECTED\"}");
    }
    
    public void mqttSubTopic(){
    	System.out.println(" =============   mqttSubTopic! ============== ");
        int cookie;
        Random random = new Random();
        cookie = random.nextInt(65535);
    	
    	String deviceId = GatewayInfo.getDeviceID();   	
		DataTransService.mqttSubTopic(cookie, "/gws/"+ deviceId +"/signaltrans/v2/categories/#",1);
    }
	
    private void getDataReportAnswer(IotaMessage iotaMsg) {
		String deviceId = iotaMsg.getString(DataTransService.DATATRANS_IE_DEVICEID);
		int retcode = iotaMsg.getUint(DataTransService.DATATRANS_IE_RESULT, 0);
		System.out.println("deviceId:" + deviceId + "data report, ret = " + retcode);
        if (deviceId.equals(GatewayInfo.getDeviceID())) {
        	System.out.println("report gateway data success, cookie = " + iotaMsg.getUint(DataTransService.DATATRANS_IE_COOKIE, 0));
        }
        if (deviceId.equals(GatewayInfo.getSensorId())) {
        	System.out.println(" report sensor data success ");
        }
    } 
    
    private void getCmdReceive(IotaMessage iotaMsg) {
    	System.out.println("=========receive iotCMD ============");
        String deviceId = iotaMsg.getString(DataTransService.DATATRANS_IE_DEVICEID);
        String requestId = iotaMsg.getString(DataTransService.DATATRANS_IE_REQUSTID);
        String serviceId = iotaMsg.getString(DataTransService.DATATRANS_IE_SERVICEID);
        String method = iotaMsg.getString(DataTransService.DATATRANS_IE_METHOD);
        String cmd = iotaMsg.getString(DataTransService.DATATRANS_IE_CMDCONTENT);
        if (method.equals("REMOVE_GATEWAY")) {
            //rmvGateway(context);
        }

        System.out.println ("Receive cmd :"
                + "\ndeviceId  = " + deviceId
                + "\nrequestId = " + requestId
                + "\nserviceId = " + serviceId
                + "\nmethod    = " + method
                + "\ncmd       = " + cmd);
    }
    
	@Override
	public void update(IotaMessage iotaMsg) {
		int mMsgType = iotaMsg.getMsgType();
		switch(mMsgType) {
		//ϱӦ
		case IodevService.IODEV_MSG_DATA_REPORT_RSP:
			getDataReportAnswer(iotaMsg);
			break;
		//
		case IodevService.IODEV_MSG_RECEIVE_CMD:
			getCmdReceive(iotaMsg);
			break;
		//MQTTϢ
		case IodevService.IODEV_MSG_MQTT_PUB_RSP:
			mqttPubRsp(iotaMsg);
			break;
		//MQTT topic	
		case IodevService.IODEV_MSG_MQTT_SUB_ACK:
			mqttSubtopicRsp(iotaMsg);
			break;			
		default:
			break;
		}
	}
	
	private void mqttPubRsp(IotaMessage iotaMsg){
		int retcode = iotaMsg.getUint(DataTransService.DATATRANS_IE_RESULT, 0);
		if(0 == retcode){
			System.out.println("mqtt Data Pub success");
		}else{
			System.out.println("mqtt Data Pub failed ,ret = " + retcode);
		}
	}
	
	private void mqttSubtopicRsp(IotaMessage iotaMsg){
		int retcode = iotaMsg.getUint(DataTransService.DATATRANS_IE_RESULT, 0);
		if(0 == retcode){
			System.out.println("mqtt sub topic success");
		}else{
			System.out.println("mqtt sub topic failed ,ret = " + retcode);
		}
	}
}
